﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace BH1792APP
{

    public partial class Form1 : Form
    {
        delegate void ShowGraph(byte[] b);

        public const int WIDTH = 5;
        public const int HIGHT = 50000;

        string ComNumber;
        byte[] dat = new byte[128];
        double count;
        bool running;

        public Form1()
        {
            InitializeComponent();
            running = false;

            // チャートの初期化設定
            initChart(chart);

            CDCcom comport = new CDCcom("VID_04D8&PID_000A");
            ComNumber = comport.getComNumber();

            if (ComNumber == null)
            {
                MessageBox.Show("CDC COM Port is not found!");
                Environment.Exit(0);
            }

            serialPort1.BaudRate = 57600;
            serialPort1.PortName = ComNumber;
            serialPort1.Open();


            serialPort1.ReadTimeout = 5000;
            serialPort1.DiscardInBuffer();

        }

        private void initChart(Chart chart)
        {

            // チャート全体の背景色を設定
            chart.BackColor = Color.MidnightBlue;
            chart.ChartAreas[0].BackColor = Color.Transparent;

            // チャート表示エリア周囲の余白をカットする
            chart.ChartAreas[0].InnerPlotPosition.Auto = false;
            chart.ChartAreas[0].InnerPlotPosition.Width = 90; // 100%
            chart.ChartAreas[0].InnerPlotPosition.Height = 90;  // 90%(横軸のメモリラベル印字分の余裕を設ける)
            chart.ChartAreas[0].InnerPlotPosition.X = 15;
            chart.ChartAreas[0].InnerPlotPosition.Y = 0;


            // X,Y軸情報のセット関数を定義
            Action<Axis> setAxis = (axisInfo) =>
            {

                // 軸のメモリラベルのフォントサイズ上限値を制限
                axisInfo.LabelAutoFitMaxFontSize = 12;

                // 軸のメモリラベルの文字色をセット
                axisInfo.LabelStyle.ForeColor = Color.White;

                // 軸タイトルの文字フォントをセット
                axisInfo.TitleFont = new Font("ＭＳ 明朝", 10, FontStyle.Regular);

                // 軸タイトルの文字色をセット
                axisInfo.TitleForeColor = Color.White;

                // 軸の色をセット
                axisInfo.LineColor = Color.White;
                axisInfo.MajorGrid.Enabled = false;
                axisInfo.MajorGrid.LineColor = Color.White;// ColorTranslator.FromHtml("#008242");

            };

            // X,Y軸の表示方法を定義
            setAxis(chart.ChartAreas[0].AxisY);
            setAxis(chart.ChartAreas[0].AxisX);

            chart.ChartAreas[0].AxisY.Minimum = 0;    // 縦軸の最小値
            chart.ChartAreas[0].AxisX.Minimum = 0;    // 横軸の最小値
            chart.ChartAreas[0].AxisY.Maximum = HIGHT;    // 縦軸の最大値
            chart.ChartAreas[0].AxisX.Maximum = WIDTH;    // 横軸の最大値

            chart.ChartAreas[0].AxisX.LineWidth = 3;
            chart.ChartAreas[0].AxisY.LineWidth = 3;
            chart.ChartAreas[0].AxisX.Interval = 1;
            chart.ChartAreas[0].AxisY.Interval = 10000;

            //           chart.AntiAliasing = AntiAliasingStyles.None;

            chart.ChartAreas[0].AxisX.Title = "時間　（ 秒 ）　";  // X軸タイトル設定
            chart.ChartAreas[0].AxisY.Title = "フォトダイオード出力　（カウント）";  // Y軸タイトル設定

            chart.ChartAreas[0].AxisY.MajorGrid.Enabled = true;
            chart.ChartAreas[0].AxisY.MajorGrid.LineColor = Color.LightGreen;

            // 折れ線グラフとして表示
            chart.Series[0].ChartType = SeriesChartType.Line;

            // 線の色を指定
            chart.Series[0].Color = Color.White;// ColorTranslator.FromHtml("#00FF00");
            chart.Series[3].Color = Color.Red;


            // 凡例を非表示,各値に数値を表示しない
            chart.Series[0].IsVisibleInLegend = false;
            chart.Series[0].IsValueShownAsLabel = false;
            chart.Series[3].IsVisibleInLegend = false;
            chart.Series[3].IsValueShownAsLabel = false;

            chart.Series[0].Points.Clear();
            chart.Series[3].Points.Clear();

            chart.Series[1].Points.Clear();
            chart.Series[1].Points.AddXY(WIDTH, 0);//グラフの上と左を閉じる
            chart.Series[1].Points.AddXY(WIDTH, HIGHT);
            chart.Series[1].Points.AddXY(0, HIGHT);
            chart.Series[1].BorderWidth = 3;//線の太さ

            chart.Series[1].ChartType = SeriesChartType.Line;
            chart.Series[1].Color = Color.White;
            chart.Series[1].IsValueShownAsLabel = false;
            chart.Series[1].IsVisibleInLegend = false;

            chart.Series[2].Color = Color.Yellow;
            chart.Series[2].IsValueShownAsLabel = false;
            chart.Series[2].IsVisibleInLegend = false;
        }

        private void showChart(byte[] b)
        {
            double num, num1;

            for (int i = 0; i < 32; i++)
            {
                int k = ((int)b[4 * i + 1] << 8) + (int)b[4 * i];
                int l = ((int)b[4 * i + 3] << 8) + (int)b[4 * i + 2];
                num = (double)k;
                num1 = (double)l;


                if (count != 1279)
                {
                    chart.Series[0].Points.AddXY(count / 256, num);
                    chart.Series[3].Points.AddXY(count / 256, num1);
                }
                else
                {
                    chart.Series[0].Points.AddXY(count / 256, Double.NaN);
                    chart.Series[3].Points.AddXY(count / 256, Double.NaN);

                    //値がない(非数)ことを示すDouble型フィールドで、これで帰線がなくなる。
                }

                count++;

                chart.Series[2].Points.Clear();

                chart.Series[2].Points.AddXY(count / 256, 0);//移動バーの描画
                chart.Series[2].Points.AddXY(count / 256, 60000);


                if (count == 1280) count = 0;

                if (chart.Series[0].Points.Count >= 1280)
                {
                    chart.Series[0].Points.RemoveAt(0);//以前のデータを消去する
                    chart.Series[3].Points.RemoveAt(0);//以前のデータを消去する
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (running == false)
            {
                dat[0] = (byte)'s';
                serialPort1.Write(dat, 0, 1);

            }
            else
            if (serialPort1.IsOpen == false)
                serialPort1.Open();

        }

        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            running = true;
            serialPort1.Read(dat, 0, 128);
            this.BeginInvoke(new ShowGraph(showChart), dat);
        }


        private void button2_Click(object sender, EventArgs e)
        {
            serialPort1.Close();
        }
    }
}
